<div align="center">
  <img src="https://res.cloudinary.com/adonisjs/image/upload/q_100/v1564392111/adonis-banner_o9lunk.png" width="600px">
</div>

# Http Server
[![circleci-image]][circleci-url] [![npm-image]][npm-url] ![][typescript-image] [![license-image]][license-url]

Node.js HTTP server with a slick router used by AdonisJs. Think of it as an ExpressJs style server written in Typescript.

<!-- START doctoc generated TOC please keep comment here to allow auto update -->
<!-- DON'T EDIT THIS SECTION, INSTEAD RE-RUN doctoc TO UPDATE -->
## Table of contents

- [Usage](#usage)
- [Router](#router)
- [Profiler action labels](#profiler-action-labels)
- [API](#api)
- [Maintainers](#maintainers)

<!-- END doctoc generated TOC please keep comment here to allow auto update -->

## Usage
Install the package from npm registry as follows:

```sh
npm i @adonisjs/http-server

# yarn
yarn add @adonisjs/http-server
```

and then use it as follows:

```ts
import proxyaddr from 'proxy-addr'
import { createServer } from 'http'
import { Logger } from '@adonisjs/logger/build/standalone'
import { Profiler } from '@adonisjs/profiler/build/standalone'
import { Server } from '@adonisjs/http-server/build/standalone'

const logger = new Logger({ enabled: false, level: 'trace', name: 'adonis' })
const profiler = new Profiler({})
const config = {
  etag: false,
  jsonpCallbackName: 'callback',
  cookie: {},
  subdomainOffset: 2,
  generateRequestId: false,
  secret: Math.random().toFixed(36).substring(2, 38),
  trustProxy: proxyaddr.compile('loopback'),
  allowMethodSpoofing: false,
}

const server = new Server({} as any, logger, profiler, config)
server.optimize()

createServer(server.handle.bind(server)).listen(3000)
```

## Router
AdonisJs has one of the most advanced and fast router. It has support for `Route groups`, `Resourceful resources` and many more.

```ts
router.group(() => {
  router.get('/', async () => {
  })
}).prefix('v1')
```

## Profiler action labels
Following is the list of actions profiled during an HTTP request

- http:request
- http:before:hooks
- http:route:match
- http:route:stack
- http:after:hooks
- http:route:handler

We recommend you to check the API docs to get a complete reference of all the classes.

## API
Following are the autogenerated files via Typedoc

* [API](docs/README.md)

## Maintainers
[Harminder virk](https://github.com/thetutlage)

[circleci-image]: https://img.shields.io/circleci/project/github/adonisjs/http-server/master.svg?style=for-the-badge&logo=circleci
[circleci-url]: https://circleci.com/gh/adonisjs/http-server "circleci"

[npm-image]: https://img.shields.io/npm/v/@adonisjs/http-server.svg?style=for-the-badge&logo=npm
[npm-url]: https://npmjs.org/package/@adonisjs/http-server "npm"

[typescript-image]: https://img.shields.io/badge/Typescript-294E80.svg?style=for-the-badge&logo=typescript

[license-url]: LICENSE.md
[license-image]: https://img.shields.io/aur/license/pac.svg?style=for-the-badge
