"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const net_1 = require("net");
const proxy_addr_1 = __importDefault(require("proxy-addr"));
const http_1 = require("http");
const helpers_1 = require("../helpers");
const Request_1 = require("../Request");
const Response_1 = require("../Response");
class HttpContext {
    constructor(request, response, logger, profiler) {
        this.request = request;
        this.response = response;
        this.logger = logger;
        this.profiler = profiler;
        this.params = {};
        this.subdomains = {};
    }
    static create(routePattern, routeParams, logger, profiler, encryption, req, res, serverConfig) {
        req = req || new http_1.IncomingMessage(new net_1.Socket());
        res = res || new http_1.ServerResponse(req);
        serverConfig = Object.assign({
            secret: Math.random().toFixed(36).substring(2, 38),
            subdomainOffset: 2,
            allowMethodSpoofing: true,
            etag: false,
            generateRequestId: false,
            cookie: {},
            jsonpCallbackName: 'callback',
            trustProxy: proxy_addr_1.default.compile('loopback'),
        }, serverConfig || {});
        req.url = req.url || helpers_1.processPattern(routePattern, routeParams);
        const request = new Request_1.Request(req, res, encryption, {
            allowMethodSpoofing: serverConfig.allowMethodSpoofing,
            subdomainOffset: serverConfig.subdomainOffset,
            trustProxy: serverConfig.trustProxy,
            generateRequestId: serverConfig.generateRequestId,
        });
        const response = new Response_1.Response(req, res, {
            etag: serverConfig.etag,
            cookie: serverConfig.cookie,
            jsonpCallbackName: serverConfig.jsonpCallbackName,
        });
        const ctx = new HttpContext(request, response, logger, profiler);
        ctx.route = {
            pattern: routePattern,
            middleware: [],
            handler: async () => 'handled',
            meta: {},
        };
        ctx.params = routeParams;
        return ctx;
    }
}
exports.HttpContext = HttpContext;
