"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class MiddlewareStore {
    constructor(container) {
        this._list = [];
        this._named = {};
        this._resolver = container.getResolver();
    }
    _resolveMiddleware(middleware) {
        return typeof (middleware) === 'function' ? {
            type: 'function',
            value: middleware,
            args: [],
        } : Object.assign(this._resolver.resolve(`${middleware}.handle`), { args: [] });
    }
    register(middleware) {
        this._list = middleware.map(this._resolveMiddleware.bind(this));
        return this;
    }
    registerNamed(middleware) {
        this._named = Object.keys(middleware).reduce((result, alias) => {
            result[alias] = this._resolveMiddleware(middleware[alias]);
            return result;
        }, {});
        return this;
    }
    get() {
        return this._list;
    }
    getNamed(name) {
        return this._named[name] || null;
    }
    async invokeMiddleware(middleware, params) {
        if (middleware.type === 'function') {
            return middleware.value(params[0], params[1], middleware.args);
        }
        const args = [params[0], params[1]];
        args.push(middleware.args);
        return this._resolver.call(middleware, undefined, args);
    }
}
exports.MiddlewareStore = MiddlewareStore;
