"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const macroable_1 = require("macroable");
const helpers_1 = require("../helpers");
class Route extends macroable_1.Macroable {
    constructor(_pattern, _methods, _handler, _globalMatchers) {
        super();
        this._pattern = _pattern;
        this._methods = _methods;
        this._handler = _handler;
        this._globalMatchers = _globalMatchers;
        this._domain = 'root';
        this._matchers = {};
        this._prefixes = [];
        this._middleware = [];
        this.deleted = false;
    }
    _getMatchers() {
        return Object.assign({}, this._globalMatchers, this._matchers);
    }
    _getPattern() {
        const pattern = helpers_1.dropSlash(this._pattern);
        const prefix = this._prefixes.slice().reverse().map((prefix) => helpers_1.dropSlash(prefix)).join('');
        return prefix ? `${prefix}${pattern === '/' ? '' : pattern}` : pattern;
    }
    where(param, matcher) {
        if (this._matchers[param]) {
            return this;
        }
        this._matchers[param] = typeof (matcher) === 'string' ? new RegExp(matcher) : matcher;
        return this;
    }
    prefix(prefix) {
        this._prefixes.push(prefix);
        return this;
    }
    domain(domain, overwrite = false) {
        if (this._domain === 'root' || overwrite) {
            this._domain = domain;
        }
        return this;
    }
    middleware(middleware, prepend = false) {
        middleware = Array.isArray(middleware) ? middleware : [middleware];
        this._middleware = prepend ? middleware.concat(this._middleware) : this._middleware.concat(middleware);
        return this;
    }
    as(name, prepend = false) {
        this.name = prepend ? `${name}.${this.name}` : name;
        return this;
    }
    namespace(namespace, overwrite = false) {
        if (!this._explicitNamespace || overwrite) {
            this._explicitNamespace = namespace;
        }
        return this;
    }
    toJSON() {
        return {
            domain: this._domain,
            pattern: this._getPattern(),
            matchers: this._getMatchers(),
            meta: {
                namespace: this._explicitNamespace,
            },
            name: this.name,
            handler: this._handler,
            methods: this._methods,
            middleware: this._middleware,
        };
    }
}
exports.Route = Route;
Route._macros = {};
Route._getters = {};
