"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const matchit_1 = __importDefault(require("matchit"));
const lodash_1 = require("lodash");
const utils_1 = require("@poppinss/utils");
class Store {
    constructor() {
        this.tree = { tokens: [], domains: {} };
    }
    _getDomainNode(domain) {
        if (!this.tree.domains[domain]) {
            this.tree.tokens.push(matchit_1.default.parse(domain));
            this.tree.domains[domain] = {};
        }
        return this.tree.domains[domain];
    }
    _getMethodRoutes(domain, method) {
        const domainNode = this._getDomainNode(domain);
        if (!domainNode[method]) {
            domainNode[method] = { tokens: [], routes: {} };
        }
        return domainNode[method];
    }
    add(route) {
        const routeJSON = lodash_1.cloneDeep(lodash_1.pick(route, [
            'pattern',
            'handler',
            'meta',
            'middleware',
            'name',
        ]));
        route.methods.forEach((method) => {
            const methodRoutes = this._getMethodRoutes(route.domain || 'root', method);
            if (methodRoutes.routes[route.pattern]) {
                throw new utils_1.Exception(`Duplicate route \`${method}:${route.pattern}\``, 500, 'E_DUPLICATE_ROUTE');
            }
            methodRoutes.tokens.push(matchit_1.default.parse(route.pattern, route.matchers));
            methodRoutes.routes[route.pattern] = routeJSON;
        });
        return this;
    }
    matchDomain(domain) {
        return matchit_1.default.match(domain || 'root', this.tree.tokens);
    }
    match(url, method, domain) {
        var _a, _b, _c, _d;
        const matchedDomain = this.tree.domains[((_b = (_a = domain) === null || _a === void 0 ? void 0 : _a.storeMatch[0]) === null || _b === void 0 ? void 0 : _b.old) || 'root'];
        if (!matchedDomain) {
            return null;
        }
        const matchedMethod = this.tree.domains[((_c = domain) === null || _c === void 0 ? void 0 : _c.storeMatch[0].old) || 'root'][method];
        if (!matchedMethod) {
            return null;
        }
        const matchedRoute = matchit_1.default.match(url, matchedMethod.tokens);
        if (!matchedRoute.length) {
            return null;
        }
        return {
            route: matchedMethod.routes[matchedRoute[0].old],
            params: matchit_1.default.exec(url, matchedRoute),
            subdomains: ((_d = domain) === null || _d === void 0 ? void 0 : _d.value) ? matchit_1.default.exec(domain.value || 'root', domain.storeMatch)
                : {},
        };
    }
}
exports.Store = Store;
